package killer_sokoban;

import java.util.HashMap;

public abstract class AbstractField {
	
	
	protected HashMap<Direction, AbstractField> neighbours=new HashMap<Direction, AbstractField>();
	protected HashMap<Direction, Boolean> gotCornerFrom = new HashMap<Direction, Boolean>();
	
	/**
	 * Konstruktor
	 */
	public AbstractField() {
		gotCornerFrom.put(Direction.UP, false);
		gotCornerFrom.put(Direction.RIGHT, false);
		gotCornerFrom.put(Direction.DOWN, false);
		gotCornerFrom.put(Direction.LEFT, false);
		
	}
	
	/**Ez a függvény azt jelzi a mező számára, hogy szeretnének rálépni.
	 * @param toward melyik irányból szeretnének rálépni
	 * @param w	munkás, aki a tolást indította
	 * @param m	mozgatható dolog a pályán, ez szeretne a mezőre lépni
	 * @return ha befogadja a paraméterül kapott Moveable-t, true-val tér vissza, ellenkező esetben false-szal.
	 */
	abstract public boolean accept(Direction toward, Worker w, Moveable m);
	
	/** Megkérdezi a mezőtől, hogy ha egy tolás következne be a paraméterként átadott irányba, akkor ez lehetséges-e.
	 * @param toward ebbe az irányba következne be a tolás
	 * @return	ha lehetséges a tolás, true-val tér vissza, egyébként false-szal.
	 */
	abstract public boolean couldMoveOn(Direction toward);
	
	/** Az adott irányba beállítja, hogy ki a mező szomszédja.
	 * @param d megadott irány
	 * @param n	a szomszéd
	 */
	public void setNeighbour(Direction d, AbstractField n)
	{
		neighbours.put(d,n);
	}
	
	/**
	 * Kiírja az adattagokat
	 */
	public void printStat()
	{

		System.out.println("neighborUp: " + Main.getFieldName(neighbours.get(Direction.UP)));
		System.out.println("neighborRight: " + Main.getFieldName(neighbours.get(Direction.RIGHT)));
		System.out.println("neighborDown: " + Main.getFieldName(neighbours.get(Direction.DOWN)));
		System.out.println("neighborLeft: " + Main.getFieldName(neighbours.get(Direction.LEFT)));
	}
	
	/**
	 * Kirajzolásért felelős függvény
	 * @param view A grafikus felület elemeinek megjelenítéséért felelős View osztály példánya
	 */
	public void shouldDraw(View view)
	{
		view.drawAbstract();
	}
	
	/**
	 * Megadja, hogy a mező saroknak számít-e
	 */
	protected boolean isCorner() {
		return gotCornerFrom.get(Direction.UP) && (gotCornerFrom.get(Direction.RIGHT) || gotCornerFrom.get(Direction.LEFT)) || gotCornerFrom.get(Direction.DOWN) && (gotCornerFrom.get(Direction.RIGHT) || gotCornerFrom.get(Direction.LEFT));
	}
	
	/**
	 * Ha szükséges, értesíti a szomszédokat arról, hogy ettől a mezőtől ők lehetnének sarkok
	 */
	public void initCorner() {
		//itt hívna egy fal minden szomszédra setToCorner()-t
	}
	
	/**
	 * Ez a függvény sarkot csinál a mezőből.
	 */
	public void setToCorner(Direction from) {
		gotCornerFrom.replace(from, true);
	}
	
	/**
	 * A sarokba betoltunk valamit, így a szomszédjaiból sarok lett.
	 */
	public void fillCorner() {
		if(!isCorner()) return;
		
		for(Direction dir : gotCornerFrom.keySet()) {
			if (gotCornerFrom.get(dir)) {
				neighbours.get(dir.opposite()).setToCorner(dir);
			}
		}
	}

}
