package killer_sokoban;

import java.io.*;
import java.util.*;
import java.util.Map.Entry;


import javafx.application.Application;
import javafx.event.EventHandler;
import javafx.geometry.Pos;
import javafx.scene.Scene;
import javafx.scene.canvas.Canvas;
import javafx.scene.input.KeyCode;
import javafx.scene.input.KeyEvent;
import javafx.scene.layout.*;
import javafx.scene.paint.Color;
import javafx.scene.text.Font;
import javafx.scene.text.Text;
import javafx.stage.Stage;



public class Main extends Application {
	
	class moveHandler implements EventHandler<KeyEvent>
	{

		@Override
		public void handle(KeyEvent ke) {
			//ha játék vége, nem lépünk..
			if (!gameRunning) return;
			
			//*
			//Turn-by-turn mode; control with arrow keys, put oil with 'O' and honey with 'H'
			//-------------------------------------------------------------------------------
			if (ke.getCode().equals(KeyCode.UP)) {
				activeWorker.step(Direction.UP);
			} else if (ke.getCode().equals(KeyCode.RIGHT)) {
				activeWorker.step(Direction.RIGHT);
			} else if (ke.getCode().equals(KeyCode.DOWN)) {
				activeWorker.step(Direction.DOWN);
			} else if (ke.getCode().equals(KeyCode.LEFT)) {
				activeWorker.step(Direction.LEFT);
			} else if (ke.getCode().equals(KeyCode.H)) {
				activeWorker.putHoney();
			} else if (ke.getCode().equals(KeyCode.O)) {
				activeWorker.putOil();
			}
			
			//swapping active worker
			if (aliveWorkers.size() != 0) {
				boolean activeFound = false;
				int i;
				for(i=0; i<aliveWorkers.size() && !activeFound; i++) {
					if(aliveWorkers.values().toArray()[i] == activeWorker) {
						activeFound = true;
					}
				}
				if(i==aliveWorkers.size()) activeWorker = (Worker)aliveWorkers.values().toArray()[0];
				else activeWorker = (Worker)aliveWorkers.values().toArray()[i];
			}
			
			/*/
			//Action mode; w1 controls with WASD, puts oil with 'E' and honey with 'Q'; w2 controls with IJKL, puts oil with 'O' and honey with 'U'
			//-------------------------------------------------------------------------------------------------------------------------------------
			if (ke.getCode().equals(KeyCode.W)) {
				workers.get("w1").step(Direction.UP);
			} else if (ke.getCode().equals(KeyCode.A)) {
				workers.get("w1").step(Direction.LEFT);
			} else if (ke.getCode().equals(KeyCode.S)) {
				workers.get("w1").step(Direction.DOWN);
			} else if (ke.getCode().equals(KeyCode.D)) {
				workers.get("w1").step(Direction.RIGHT);
			} else if (ke.getCode().equals(KeyCode.Q)) {
				if(aliveWorkers.containsKey("w1")) {
					workers.get("w1").putHoney();
				}
			} else if (ke.getCode().equals(KeyCode.E)) {
				if(aliveWorkers.containsKey("w1")) {
					workers.get("w1").putOil();
				}
			}
			else if (ke.getCode().equals(KeyCode.I)) {
				workers.get("w2").step(Direction.UP);
			} else if (ke.getCode().equals(KeyCode.J)) {
				workers.get("w2").step(Direction.LEFT);
			} else if (ke.getCode().equals(KeyCode.K)) {
				workers.get("w2").step(Direction.DOWN);
			} else if (ke.getCode().equals(KeyCode.L)) {
				workers.get("w2").step(Direction.RIGHT);
			} else if (ke.getCode().equals(KeyCode.U)) {
				if(aliveWorkers.containsKey("w2")) {
					workers.get("w2").putHoney();
				}
			} else if (ke.getCode().equals(KeyCode.O)) {
				if(aliveWorkers.containsKey("w2")) {
					workers.get("w2").putOil();
				}
			}
			//*/
			
			//checking if game should end
			checkGameEnded();
			
			view.update();
		}
		
	}
	
	private static HashMap<String, AbstractField> map = new HashMap<>(); //itt tároljuk a pályán lévő mezőket név (pl. "f_2_4") és referencia szerint
	private static HashMap<String, Worker> workers = new HashMap<>(); //a munkások
	private static HashMap<String, Crate> crates = new HashMap<>(); //a ládák - azért szedtem így külön mindent, hogy a játék vége detektálásánál végig lehessen kérdezgetni csak a ládákat - comment by SF
	
	private static HashMap<String, Worker> aliveWorkers = new HashMap<>();
	
	private static Worker activeWorker;
	private static boolean gameRunning;
	
	//A beolvasáshoz kellenek. Azért tettem ide őket, hogy a parseMap függvény is lássa őket.
	private static InputStreamReader isr =	new InputStreamReader(System.in);
	private static BufferedReader br = new BufferedReader(isr);
	
	private static View view=new View();
	
	/**
	 * Játék kezdetéhez szükséges előkészüleletek
	 */
	@Override
	public void start(Stage primaryStage) throws Exception {
		// Ablak felépítése
		primaryStage.setTitle("Killer Sokoban - SPoF");
        BorderPane root = new BorderPane();
        root.setStyle("-fx-background-color: #505050");
        root.setStyle("-fx-font-family: 'Rockwell Condensed', Consolas, sans-serif");
        root.setStyle("-fx-text-fill: #ffffff");
        
        //Canvas holder felépítése
		StackPane mapHolder = new StackPane();
		mapHolder.setAlignment(Pos.CENTER);
		mapHolder.setStyle("-fx-background-color: #505050");
		root.setCenter(mapHolder);
        
        //Vászon felépítése
        Canvas canvas = new Canvas(750, 550);
        view.setCanvas(canvas);
        mapHolder.getChildren().add(canvas);
        
        //Game over screen
		view.gameOverContainer = new GridPane();
		view.gameOverContainer.setVisible(false);
		view.gameOverContainer.setAlignment(Pos.CENTER);
		view.gameOverContainer.setStyle("-fx-background-color: #00000099");
		mapHolder.getChildren().add(view.gameOverContainer);
		
		VBox gameOverBox = new VBox();
		gameOverBox.setAlignment(Pos.CENTER);
		gameOverBox.setSpacing(5);
		view.gameOverContainer.add(gameOverBox,0,0);
		
		Text gameOverTitle = new Text("Game over");
		gameOverTitle.setFill(Color.WHITE);
		gameOverTitle.setFont(Font.font("Rockwell", 34));
		gameOverTitle.setUnderline(true);
		gameOverBox.getChildren().add(gameOverTitle);
		
		view.w1FinalScore = new Text("w1: 0 points");
		view.w1FinalScore.setFont(Font.font("Rockwell", 26));
		view.w1FinalScore.setFill(Color.WHITE);
		gameOverBox.getChildren().add(view.w1FinalScore);
		view.w2FinalScore = new Text("w2: 0 points");
		view.w2FinalScore.setFont(Font.font("Rockwell", 26));
		view.w2FinalScore.setFill(Color.WHITE);
		gameOverBox.getChildren().add(view.w2FinalScore);
        
        //Állapotsor felépítése
		HBox pointsContainer = new HBox();
		pointsContainer.setStyle("-fx-background-color: #404040; -fx-font-size: 10pt");
		root.setBottom(pointsContainer);
		view.pointBar = new Text("w1: 0 points\tw2: 0 points");
		view.pointBar.setFill(Color.WHITE);
		pointsContainer.getChildren().add(view.pointBar);
		
        
        //Ablak megjelenítése
        primaryStage.setScene(new Scene(root));
        primaryStage.show();
        
        //Event handlerek és egyebek
        primaryStage.addEventHandler(KeyEvent.KEY_RELEASED, new moveHandler());
        
        //Itt kellene betölteni a pályát
		runCommand("script Testmap1.in");
		
		//Egyéb adminisztratív teendők...
		activeWorker = workers.get("w1");
		gameRunning=true;
		
		//Ez végzi a rajzolást
		view.update();
		
		
	}
	
	/**
	 * Kirajzolja a megadott koordinátákra a megfelelő mezőt
	 * @param y y koordináta
	 * @param x x koordináta
	 */
	public static void drawRequiredFor(int y, int x)
	{
		AbstractField af=map.get("f_"+y+"_"+x);
		af.shouldDraw(view);
	}
	
	public static void main(String[] args) throws IOException {
		launch(args);
	}
	
	public static String getFieldName(AbstractField field)
	{
		return getKeyByValue(map, field);
	}
	
	public static String getMoveableName(Moveable m)
	{
		HashMap<String, Moveable> moveables = new HashMap<>(workers);
		moveables.putAll(crates);
		
		return getKeyByValue(moveables, m);
	}
	
	public static int getWorkerPoint(String wName) { return workers.get(wName).getPoints(); }
	
	public static void deleteWorker(Worker w)
	{
		aliveWorkers.remove(getKeyByValue(workers, w));
	}
	
	public static void deleteCrate(Crate c)
	{
		crates.remove(getKeyByValue(crates, c));
	}
	
	/**
	 * Ellenőrzi, hogy vége-e a játéknak
	 */
	private static void checkGameEnded() {
		if (aliveWorkers.size() == 0) {
			endGame();
			return;
		}
		
		boolean ended = true;
		for(Crate c : crates.values()) {
			if(!c.couldGameFinish()) ended = false;
		}
		
		if (ended) {
			endGame();
		}
	}
	
	/**
	 * Jelzi, hogy vége a játéknak
	 */
	private static void endGame() {
		gameRunning=false;
	}
	
	/**
	 * Visszaadja, hogy megy-e még a játék
	 * @return
	 */
	public static boolean isGameRunning() { return gameRunning; }
	
	private static void runCommand(String cmdline) throws NumberFormatException, IOException
	{
		//Ha üres sort kaptunk, ignoráljuk
		//Emiatt áttekinthetőbb bemeneteket lehet csinálni
		if (cmdline.equals("")) return;
		
		
		//Parancs részekgre tördelése
		String cmd[];
		cmd = cmdline.split(" ");
		
		
		//Parancs értelmezés, és a szükséges függvény meghívása
		if (cmd[0].equals("parsemap")) {			
			parseMap(Integer.parseInt(cmd[1]), Integer.parseInt(cmd[2]));
		}
		else if (cmd[0].equals("worker")) {
			spawnWorker(cmd[1],Double.parseDouble(cmd[2]),cmd[3]);
		}
		else if (cmd[0].equals("crate")) {
			spawnCrate(cmd[1],cmd[2]);
		}
		else if (cmd[0].equals("setswitch")) {
			setSwitch(cmd[1], cmd[2]);
		}
		else if (cmd[0].equals("putfluid")) {
			putFluid(cmd[1], cmd[2].charAt(0));
		}
		else if (cmd[0].equals("step")) {
			step(cmd[1],cmd[2]);
		}
		else if (cmd[0].equals("stat")) {
			stat(cmd[1]);
		}
		else if (cmd[0].equals("setcorner")) {
			setCorner(cmd[1], cmd[2]);
		}
		else if (cmd[0].equals("script")) {
			script(cmd[1]);
		}
		else 
		{
			System.out.println("unrecognized command");
		}
		
	}
	
	/**
	 * Létrehozza a pályát
	 * @param lines beolvasott sorok száma
	 * @param columns beolvasott oszlopok száma
	 * @throws IOException
	 */
	private static void parseMap(int lines, int columns) throws IOException {
		//Tisztítás
		map.clear();
		workers.clear();
		crates.clear();
		
		//Feldolgozás
		//Mezők létrehozása
		for (int i=0; i<lines; i++)
		{
			String line= br.readLine();
			for (int j=0; j<columns; j++)
			{
				String name="f_"+i+"_"+j; //név generálása
				switch (line.charAt(j)) {
					case 'w' :
						map.put(name, new Wall());
						break;
					case 'm' :
						map.put(name, new SimpleField());
						break;
					case 'h' :
						map.put(name, new Hole());
						break;
					case 'k' :
						map.put(name, new SwitchField());
						break;
					case 's' :
						map.put(name, new SwitchableHole());
						break;
					case 't' :
						map.put(name, new TargetField());
						break;
				}
			}
		}
		
		
		//Szomszédságok beállítása
		//Minden mezőn végigmegyünk. Ha létezik a megfelelő irányú szomszédja, beállítjuk.
		for (int i=0; i<lines; i++)
		{
			for (int j=0; j<columns; j++)
			{
				String name="f_"+i+"_"+j; //név generálása
				String nameUp="f_"+(i-1)+"_"+j; //név generálása
				String nameRight="f_"+i+"_"+(j+1); //név generálása
				String nameDown="f_"+(i+1)+"_"+j; //név generálása
				String nameLeft="f_"+i+"_"+(j-1); //név generálása
				
				if (map.containsKey(nameUp))
					map.get(name).setNeighbour(Direction.UP, map.get(nameUp));
				if (map.containsKey(nameRight))
					map.get(name).setNeighbour(Direction.RIGHT, map.get(nameRight));
				if (map.containsKey(nameDown))
					map.get(name).setNeighbour(Direction.DOWN, map.get(nameDown));
				if (map.containsKey(nameLeft))
					map.get(name).setNeighbour(Direction.LEFT, map.get(nameLeft));
			}
		}
		
		//Sarkok inicializálása
		initCorners();
		
		//View újrainicializálása
		view.init(lines,  columns);
	}
	
	/**
	 * Létrehoz egy munkást a megadott paraméterekkel, hozzáadja workers adattaghoz, és ráállítja a megfelelő mezőre
	 * @param name munkás neve
	 * @param force munkás ereje
	 * @param field munkás mezője
	 */
	private static void spawnWorker(String name, double force, String field)
	{
		Worker w=new Worker(force);
		workers.put(name, w);
		aliveWorkers.put(name, w);
		AbstractField f=map.get(field);
		f.accept(Direction.RIGHT, w, w);
	}
	
	/**
	 * Létrehoz egy ládát, hozzáadja a crates adattaghoz, és ráállítja a megfelelő mezőre.
	 * @param name láda neve
	 * @param field láda mezője
	 */
	private static void spawnCrate(String name, String field)
	{
		Crate c=new Crate();
		crates.put(name, c);
		AbstractField f=map.get(field);
		f.accept(Direction.RIGHT, new Worker(0), c);
	}
	
	/**
	 * A megadott SwitchField-et ráállítja a megadott SwitchableHole-ra
	 * @param shole SwitchableHole
	 * @param mySwitch SwitchField
	 */
	private static void setSwitch(String shole, String mySwitch)
	{
		((SwitchField) map.get(mySwitch)).setSHole((SwitchableHole) map.get(shole));
	}
	
	/**
	 * A megadott mezőre mézet helyez, ha a fluid ‘h’ és olajat, ha az ‘o’
	 * @param field megadott mező
	 * @param fluid olaj vagy méz
	 */
	private static void putFluid(String field, char fluid)
	{
		SimpleField sf=((SimpleField) map.get(field));
		if (fluid=='h')
			sf.putHoney();
		else if (fluid=='o')
			sf.putOil();
		else
			System.out.println("you can only choose either 'h' or 'o'");
	}
	
	/**
	 * A megadott munkást a megadott irányba próbálja léptetni
	 * @param worker megadott munkás
	 * @param toward merre lépjen
	 */
	private static void step(String worker, String toward)
	{
		Worker w= (Worker) workers.get(worker);
		w.step(Direction.fromString(toward));
	}
	
	/**
	 *  Megkeresi, hogy milyen pályán lévő dolog nevét kapta paraméterül
	 * @param object pályán lévő dolog
	 */
	private static void stat(String object)
	{
		if (map.containsKey(object))
		{
			map.get(object).printStat();
		}
		else if (workers.containsKey(object))
		{
			workers.get(object).printStat();
		}
		else if (crates.containsKey(object))
		{
			crates.get(object).printStat();
		}
		else
		{
			System.out.println("This doesn't exist!");
		}
	}
	
	/**
	 * Elindít egy inicializálást arról, hogy mely mezők vannak sarokban
	 */
	private static void initCorners() {
		for(AbstractField f : map.values()) f.initCorner();
	}
	
	/**
	 * Sarkokat beállítja a megadott mezőn
	 * @param field megadott mező
	 */
	private static void setCorner(String field, String from) {
		map.get(field).setToCorner(Direction.fromString(from));
	}
	
	/**
	 * Beolvassa a fájlnévvel megadott fájlt, és soronként lefuttatja azt a runCommand() függvénnyel
	 * @param file fájlnév
	 * @throws IOException
	 */
	private static void script(String file) throws IOException
	{
		BufferedReader backup = br;
		
		//Maybe this works...
		File scriptFile=new File(file);
		FileInputStream fis=new FileInputStream(scriptFile);
		InputStreamReader isr =	new InputStreamReader(fis);
		br = new BufferedReader(isr);
		
		while (true) {
			String line;
			line = br.readLine();

			if (line == null) break;
			
			runCommand(line);			
		}
		
		br.close();
		
		br = backup;
		  
	}
	
	//code copied from StackOverFlow
	private static <T, E> T getKeyByValue(Map<T, E> map, E value) {
	    for (Entry<T, E> entry : map.entrySet()) {
	        if (Objects.equals(value, entry.getValue())) {
	            return entry.getKey();
	        }
	    }
	    return null;
	}

}
