package killer_sokoban;


public abstract class Moveable {
	
	protected SimpleField myField=null;
	
	/**
	 * Az objektum összenyomódásra kényszerül.
	 * @return true ha az objektum összenyomódott
	 */
	protected boolean squeez()
	{

		return false;
	}
	
	
	/**
	 * A Moveable elveszik és nem vesz részt a további játékban.
	 */
	public void kill()	{
		if (myField!=null) myField.dropMoveable();
	}
	
	//Ezek a függvények szándékosan üresek, csak a kiírást végzik
	public void stepOnTarget(Worker w)	{} 
	public void stepOffTarget(Worker w)	{}
	public void setCouldMove(boolean b) {}
	public void onSwitch(SwitchField s) {}
	
	/**Az objektum másik mezőre került.
	 * 	
	 * @param here a mező, ahova az objektum került
	 */
	public void move(SimpleField here) {

		if (myField!=null) myField.dropMoveable();
		myField=here;

	}
		
	/**Akkor hívandó, ha az m Moveable helyére jelen Moveable akar kerülni.
	 * 
	 * @param toward melyik irányból szeretnének rálépni
	 * @param w munkás, aki a tolást indította
	 * @param m mozgatható dolog a pályán, ennek a helyére akar egy másik Moveable kerülni
	 * @return ha sikerült eltolni m-et igazzal tlr vissza, ellenkező esetben hamissal
	 */
	public boolean meetWith(Direction toward, Worker w, Moveable m) {
		boolean ret=m.push(toward, w, this);

		return ret;
	}
	
	/**Akkor hívandó, ha a Moveable-t meg akarjuk tolni.
	 * 
	 * @param toward melyik iránya szeretne menni a Moveable
	 * @param w munkás, aki a tolást indította
	 * @param m az a Moveable, aki megtolja a jelenlegi Moveable-t.
	 * @return akkor tér vissza igazzal, ha távozni tudott a jelenlegi Moveable a mezőjéről.
	 */
	public boolean push(Direction toward, Worker w, Moveable m) {
	

		//trying to step away
		boolean ret=myField.stepMe(toward, w);
		
		//if couldn't step, squeezing
		boolean couldSqueez = false;
		if (!ret) couldSqueez = squeez();
		
		//finishing up

		return ret || couldSqueez;
	}
	
	/**
	 * Kiírja az adattagokat
	 */
	public void printStat()
	{
		System.out.println("myField: " + Main.getFieldName(myField) );
	}
	
	/**
	 * Kirajzolásért felel
	 * @param view
	 */
	public void shouldDraw(View view)
	{
	}
	
	/**
	 * Azt jelzi a Moveable számára, hogy sarokba érkezett. Szándékosan üres, a láda felüüldefiniálja.
	 */
	public void arriveToCorner() {}
	
}
