package killer_sokoban;

import java.util.ArrayList;

public class SimpleField extends AbstractField {
	
	protected Moveable myMoveable=null;
	protected Fluid fluid = new Fluid();

	/**Ez a függvény azt jelzi a mező számára, hogy szeretnének rálépni.
	 * @param toward melyik irányból szeretnének rálépni
	 * @param w	munkás, aki a tolást indította
	 * @param m	mozgatható dolog a pályán, ez szeretne a mezőre lépni
	 * @return ha befogadja a paraméterül kapott Moveable-t, true-val tér vissza, ellenkező esetben false-szal.
	 */
	@Override
	public boolean accept(Direction toward, Worker w, Moveable m) {
		boolean ret=true;		//Alapvetően elfogadnánk....
		if (myMoveable != null)	//...kivétel ha van rajta valaki...
		{
			ret=m.meetWith(toward, w, myMoveable); //...aki nem hajlandó elmenni
		}
		
		//Elfogadás adminisztrációja
		if (ret)
		{
			m.move(this); //Fontos a sorrend, mert ez a függvény eldobatja magát m mezőjével.
			myMoveable=m;
			if (isCorner()) m.arriveToCorner();
		}
		

		return ret;
	}

	
	/**
	 * Akkor hívja a mezőn a rajta álló Moveable, ha  egy másik mezőre szeretne lépni.
	 * @param toward  melyik irányban lévő szomszédos mezőre lép a Moveable
	 * @param w munkás, aki a mozgást indította
	 * @return ha a lépés sikeres, igazzal, ellenkzeő esetben hamissal tér vissza.
	 */
	public boolean stepMe(Direction toward, Worker w)
	{
	
		if(!w.decrementForce(fluid.getFriction())) {

			return false;
		}
		boolean ret=neighbours.get(toward).accept(toward, w, myMoveable);

		return ret;
		
	}
	
	/**
	 * Ellenőrzi, hogy van-e legalább két nem szemközti szomszédja, ahonnan
	 * egy esetleg rajta lévő láda nem lenne közvetlenül tolható. 
	 *
	 * @param d az irány ahonnan a tolás indításra került
	 */
	protected void determineBlocked(Direction d)
	{

		ArrayList<Boolean> couldMove=new ArrayList<Boolean>();
		for (int i=0; i<4; i++)
		{
			if (Direction.Dir(i)!=d.opposite()) couldMove.add(neighbours.get(d).couldMoveOn(d));
			else couldMove.add(true);
		}
		
		boolean blocked=false;
		couldMove.add(couldMove.get(0)); //körbenforgás miatt, ha az első és az utolsó irány lenne a "sarok"
		for (int i=0; i<4; i++)
		{
			if (!couldMove.get(i) && !couldMove.get(i+1)) blocked=true;
		}

				

	}
	
	
	/** Megkérdezi a mezőtől, hogy ha egy tolás következne be a paraméterként átadott irányba, akkor ez lehetséges-e.
	 * @param toward ebbe az irányba következne be a tolás
	 * @return	ha lehetséges a tolás, true-val tér vissza, egyébként false-szal.
	 */
	@Override
	public boolean couldMoveOn(Direction toward) {

		boolean ret = myMoveable==null || neighbours.get(toward).couldMoveOn(toward);

		return ret;
	}
	
	/**
	 * Eldobja a Moveable-t, az a mező felszabadul.
	 */
	public void dropMoveable()
	{
		myMoveable=null;
	}
	
	/**
	 * Kirajzolásért felel
	 */
	@Override
	public void shouldDraw(View view)
	{
		view.drawSimpleField();
		fluid.shouldDraw(view);
		if (myMoveable!=null) myMoveable.shouldDraw(view);
		super.shouldDraw(view);
	}
	
	
	protected class Fluid {
		protected double friction;
		
		Fluid() {
			friction = 1;
		}
		
		public double getFriction() {
			return friction;
		}
		
		@Override
		public String toString() {
			return "no fluid";
		}
		
		public void shouldDraw(View view)
		{
		}
		
	}
	
	protected class Oil extends Fluid {
		Oil() {
			friction = 0;
		}
		@Override
		public String toString() {
			return "oil";
		}
		@Override
		public void shouldDraw(View view)
		{
			super.shouldDraw(view);
			view.drawOil();
		}
	}
	
	protected class Honey extends Fluid {
		Honey() {
			friction = 2;
		}
		@Override
		public String toString() {
			return "honey";
		}
		@Override
		public void shouldDraw(View view)
		{
			super.shouldDraw(view);
			view.drawHoney();
		}
	}
	
	/**
	 * olaj lerakása
	 */
	public void putOil() {
		fluid = new Oil();
	}
	
	/**
	 * méz lerakása
	 */
	public void putHoney() {
		fluid = new Honey();
	}
	
	/**
	 * adattagok kiírása
	 */
	public void printStat()
	{
		super.printStat();
		System.out.println("moveable: " + Main.getMoveableName(myMoveable));
		System.out.println("fluid: " + fluid.toString());

	}
	
	/**
	 * Ez a függvény sarkot csinál a mezőből.
	 */
	@Override
	public void setToCorner(Direction from) {
		super.setToCorner(from);
		if(myMoveable != null) myMoveable.arriveToCorner();
	}
}
