package killer_sokoban;

import javafx.scene.canvas.Canvas;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.image.Image;
import javafx.scene.layout.GridPane;
import javafx.scene.paint.Color;
import javafx.scene.text.Text;

public class View {
	
	private int line, column;
	private int y, x;
	private GraphicsContext gc;
	
	private int tileSize=32;
	
	private Image honeyImg=new Image("file:GIMPimages/Honey.png");
	private Image oilImg=new Image("file:GIMPimages/Oil.png");
	private Image crateImg=new Image("file:GIMPimages/Crate.png");
	private Image holeImg=new Image("file:GIMPimages/Hole.png");
	private Image simpleFieldImg=new Image("file:GIMPimages/SimpleField.png");
	private Image switchFieldImg=new Image("file:GIMPimages/SwitchField.png");
	private Image targetFieldImg=new Image("file:GIMPimages/TargetField.png");
	private Image wallImg=new Image("file:GIMPimages/Wall.png");
	
	Text pointBar; //A pontokat ide írjuk ki.
	GridPane gameOverContainer;
	Text w1FinalScore;
	Text w2FinalScore;
	 
	/**
	 * A megjelenítendő pálya paramétereinek megadása
	 * @param line pálya sorainak száma
	 * @param column pálya oszlopainak a száma
	 */
	public void init(int line, int column)
	{
		this.line=line;
		this.column=column;
		
		gc.getCanvas().setWidth(tileSize*column);
		gc.getCanvas().setHeight(tileSize*line);
	}
	
	/**
	 * vászon beállítása, amire a View rajzol
	 * @param c vászon
	 */
	public void setCanvas(Canvas c)
	{
		this.gc=c.getGraphicsContext2D();		
	}
	
	/**
	 * A pálya minden elemének újrarajzolása
	 */
	public void update()
	{
		//gc.clearRect(0, 0, column*tileSize, line*tileSize);
		//Pálya kifestése a simplefield mező színével.
		gc.setFill(Color.rgb(92, 92, 92));
		gc.fillRect(0, 0, column*tileSize, line*tileSize);
		for (x=0;x<column;x++)
			for (y=0;y<line;y++)
				Main.drawRequiredFor(y,x);
		
		//pontok frissítése
		pointBar.setText("w1: " + Main.getWorkerPoint("w1") +" points\tw2: " + Main.getWorkerPoint("w2") + " points");
		
		//game over megjelenítése
		if (!Main.isGameRunning()) {
			System.out.println("game finished");
			w1FinalScore.setText("w1: " + Main.getWorkerPoint("w1") + " points");
			w2FinalScore.setText("w2: " + Main.getWorkerPoint("w2") + " points");
			gameOverContainer.setVisible(true);
		}
	}
		 
	/**
	 * Az x, y koordinátákkal meghatározott mezőre az Abstractfieldnek megfelelő grafikus elem kirajzolása
	 */
	 public void drawAbstract() {
		 gc.setLineWidth(0.1);
		 gc.setStroke(Color.BLUE);
		 gc.strokeRect(x*tileSize, y*tileSize, tileSize, tileSize);
	 }
	 
	 /**
	  * Az x, y koordinátákkal meghatározott mezőre a SimpleField-nek megfelelő grafikus elem kirajzolása
	  */
	 public void drawSimpleField() {
		 //gc.drawImage(simpleFieldImg, x*tileSize, y*tileSize);
	 }
	 
	 /**
	  * Az x, y koordinátákkal meghatározott mezőre a TargetField-nek megfelelő grafikus elem kirajzolása
	  */
	 public void drawTargetField() {
		 gc.drawImage(targetFieldImg, x*tileSize, y*tileSize);
	 }
	 
	 /**
	  * Az x, y koordinátákkal meghatározott mezőre a Hole-nak megfelelő grafikus elem kirajzolása
	  */
	 public void drawHole() {
		 gc.drawImage(holeImg, x*tileSize, y*tileSize);
	 }
	 
	 /**
	  * Az x, y koordinátákkal meghatározott mezőre a SwitchField-nek megfelelő grafikus elem kirajzolása
	  */
	 public void drawSwitchField() {
		 gc.drawImage(switchFieldImg, x*tileSize, y*tileSize);
	 }
	 
	 /**
	  * Az x, y koordinátákkal meghatározott mezőre a Wall-nek megfelelő grafikus elem kirajzolása
	  */
	 public void drawWall() {
		 gc.drawImage(wallImg, x*tileSize, y*tileSize);
	 }
	 
	 /**
	  * Az x, y koordinátákkal meghatározott mezőre a Oil-nak megfelelő grafikus elem kirajzolása (a már ott lévő tetejére, amely még kilátszik alóla)
	  */
	 public void drawOil() {
		 gc.drawImage(oilImg, x*tileSize, y*tileSize);
	 }
	 
	 /**
	  * Az x, y koordinátákkal meghatározott mezőre a Honey-nak megfelelő grafikus elem kirajzolása (a már ott lévő tetejére, amely még kilátszik alóla)
	  */
	 public void drawHoney() {
		 gc.drawImage(honeyImg, x*tileSize, y*tileSize);
	 }
	 
	 
	 /**
	  * Az x, y koordinátákkal  meghatározott mezőre a Worker-nek megfelelő grafikus elem kirajzolása
	  * @param w a kirajzolandó munkás
	  */
	 public void drawWorker(Worker w) {
		 String name=Main.getMoveableName(w);
		 Image wImg=new Image("file:GIMPimages/"+name+".png");
		 gc.drawImage(wImg, x*tileSize, y*tileSize);
	}
	 
	 /**
	  * Az x, y koordinátákkal meghatározott mezőre a Crate-nek megfelelő grafikus elem kirajzolása
	  */
	 public void drawCrate() {
		 gc.drawImage(crateImg, x*tileSize, y*tileSize);
	 }
	 
	 
}
